/**
 * MT4/MT5 Custom Programming. Contact me at: kolier.li@gmail.com
 */


//---------- Property ----------//
#property copyright "Copyright 2011, KoliEr Li."
#property link      "http://kolier.li"
// Client: Eric Gerry
// Tags: 


/* Change Logs */
/*
1. 1.0.1(2011-11-06) Initiate version.
2. 1.0.2(2011-12-21) Fix LotsAdd function.
                     Modify MaxOrders.
3. 1.0.3(2011-12-21) Fix typo.
4. 1.1.4(2012-02-03) Add multiple MA pairs for entry.
5. 1.2.5(2012-02-03) Add LotsAdd_SameSide option to control LotsAdd logic.
6. 1.3.6(2012-02-03) Add TakeProfitAdd, Change LotsAdd_SameSide to DD_SameSide.
                     Add multiple MA pairs for exit.
7. 1.3.7(2012-02-05) Fix bugs.
8. 1.4.8(2012-04-04) Reset all the signal if the new closed order make the currently opening amount be (MaxOrder - 1).
9. 1.5.9(2012-06-27) Feature Fix: TrailStop stealth mode typo.
                     Feature Add: Divide PipStep to PipStepProfit and PipStepLoss separately.
                     Feature Add: Add PipStepClosedProfit and PipStepClosedLoss.
                     Feature Add: PipStepPhaseOpenPrice. PipStep related order open price reference to the oldest OrderOpenPrice.
                     Feature Add: TakeProfitLimit on/off limits to the oldest takeprofit.
                        As your PTP. Only apply when this TP is valid to open order, or not EA will use traditional TP Add.
                     Feature Modify: LotsAdd from add every time to only add once and used by all subsequent orders.
1. 1.6 (2012-09-17) Fix pipstep.
                    Fix DD_SameSide lots, Takeprofit_Add.
0. 1.7 (2013-03-05) Feature Create: Ghost Stoploss.
1.     (2013-03-07) Mod (Add candidate, but get it comment out): Ghost Stoploss.
                      Should reset follow reset function.
0. 1.8 (2013-04-01) New: DD_MapGhost.
                        DD orders map to the #1 New order to operate.
0. 1.9 (2013-06-08) New: DD_MapRT.
1.     (2013-08-20) Mod: GV name for DD_MapRT.
                    Fix: DD_MapRT in tradeSell().
                    
       (2017-08-05) New: trailing stop loss expanded to three setups                    
*/

//+------------------------------------------------------------------+
//| Includes                                                         |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| Universal Constants                                              |
//+------------------------------------------------------------------+
#define TRADEPEND  -1
#define TRADEALL   0
#define TRADETRADE 1
#define TRADEBUY   1
#define TRADESELL  -1

#define PHASE_NONE 0
#define PHASE_BUY 1
#define PHASE_SELL -1

#define GET 0
#define SET 1
#define DEL -1

//+------------------------------------------------------------------+
//| User input variables                                             |
//+------------------------------------------------------------------+
extern string AdvisorName                     = "KOLIER V-13 M'GALE THRU PK. LTS. MULT."; // For Logging
extern string AdvisorVersion                  = "1.9-1"; // The version number of this script

// Entry & Exit
extern string Entry_Exit                      = "Entry & Exit";
extern bool   Entry_AtBarOpen                 = false;              // Whether to wait for new bar to confirm the signal for entry
extern int    MaxOrders                       = 10;                 // Set to 0 to disable.

extern bool   PipStepPhaseOpenPrice           = false;              // Whether to always use the first order of the phase to check the pip step.
extern double PipStepProfitSame               = 0;                  // In Pips. Profit limit range, use 0 to turn it off.
extern double PipStepLossSame                 = 0;                  // In Pips. Use 0 to turn it off.
extern double PipStepProfitOpp                = 0;                  // In Pips. Profit limit range, use 0 to turn it off.
extern double PipStepLossOpp                  = 0;                  // In Pips. Use 0 to turn it off.
extern double PipStepClosedProfit             = 0;                  // In Pips. Profit limit range, use 0 to turn it off.
extern double PipStepClosedLoss               = 0;                  // In PIps. Use 0 to turn it off.
extern double PipStepClosedProfitOpp          = 0;                  // In Pips. Profit limit range, use 0 to turn it off.
extern double PipStepClosedLossOpp            = 0;                  // In PIps. Use 0 to turn it off.

extern bool   Alternate                       = false;
extern bool   Sameternate                     = false;
extern bool   Oppternate                      = false;
extern double MaxSpread                       = 10;                 // In Pips. Prevent high volatility
extern bool   DD_SameSide                     = false;              // Control LotsAdd, TakeProfitAdd.
extern bool   DD_MapRT                        = false;              // Map the RT#1 order as DD order.
extern bool   Ghost_Stoploss                  = false;              // If use Ghost Stoploss feature.
extern bool   DD_MapGhost                     = false;              // DD orders map to the #1 New order to operate.

// @array
extern int BuyMinDiff                      = "0";                // In pips. The minimum difference between MAs for Cross to count
// iMA
extern string BuyMAFast_iMA                = "--- Buy Fast MA Settings ---";
extern int BuyMAFast_Period                = "10";               // MA Period
extern int BuyMAFast_Shift                 = "0";                // MA Shift
extern int BuyMAFast_Method                = "2";                // 0=sma, 1=ema, 2=smma, 3=lwma
extern int BuyMAFast_Price                 = "0";                // 0=close, 1=open, 2=high, 3=low, 4=median, 5=typical, 6=weighted
// iMA
extern string BuyMASlow_iMA                = "--- Buy Slow MA Settings ---";
extern int BuyMASlow_Period                = "15";               // MA Period
extern int BuyMASlow_Shift                 = "0";                // MA Shift
extern int BuyMASlow_Method                = "2";                // 0=sma, 1=ema, 2=smma, 3=lwma
extern int BuyMASlow_Price                 = "0";                // 0=close, 1=open, 2=high, 3=low, 4=median, 5=typical, 6=weighted

// ExitBuy
extern bool   ExitBuy_On                   = false;              // If turn on, will exit buy if encouter sell signal
// @array
extern int ExitBuyMinDiff                  = "0";                // In pips. The minimum difference between MAs for Cross to count
// iMA
extern string ExitBuyMAFast_iMA            = "--- Exit Buy Fast MA Settings ---";
extern int ExitBuyMAFast_Period            = "0";                // MA Period
extern int ExitBuyMAFast_Shift             = "0";                // MA Shift
extern int ExitBuyMAFast_Method            = "0";                // 0=sma, 1=ema, 2=smma, 3=lwma
extern int ExitBuyMAFast_Price             = "0";                // 0=close, 1=open, 2=high, 3=low, 4=median, 5=typical, 6=weighted
// iMA
extern string ExitBuyMASlow_iMA            = "--- Exit Buy Slow MA Settings ---";
extern int ExitBuyMASlow_Period            = "0";                // MA Period
extern int ExitBuyMASlow_Shift             = "0";                // MA Shift
extern int ExitBuyMASlow_Method            = "0";                // 0=sma, 1=ema, 2=smma, 3=lwma
extern int ExitBuyMASlow_Price             = "0";                // 0=close, 1=open, 2=high, 3=low, 4=median, 5=typical, 6=weighted

// @array
extern int SellMinDiff                     = "0";                // In pips. The minimum difference between MAs for Cross to count
// iMA
extern string SellMAFast_iMA               = "--- Sell Fast MA Settings ---";
extern int SellMAFast_Period               = "10";               // MA Period
extern int SellMAFast_Shift                = "0";                // MA Shift
extern int SellMAFast_Method               = "2";                // 0=sma, 1=ema, 2=smma, 3=lwma
extern int SellMAFast_Price                = "0";                // 0=close, 1=open, 2=high, 3=low, 4=median, 5=typical, 6=weighted
// iMA
extern string SellMASlow_iMA               = "--- Sell Slow MA Settings ---";
extern int SellMASlow_Period               = "15";               // MA Period
extern int SellMASlow_Shift                = "0";                // MA Shift
extern int SellMASlow_Method               = "2";                // 0=sma, 1=ema, 2=smma, 3=lwma
extern int SellMASlow_Price                = "0";                // 0=close, 1=open, 2=high, 3=low, 4=median, 5=typical, 6=weighted

// ExitSell
extern bool   ExitSell_On                  = false;              // If turn on, will exit buy if encouter buy signal
// @array
extern int ExitSellMinDiff                 = "0";                // In pips. The minimum difference between MAs for Cross to count
// iMA
extern string ExitSellMAFast_iMA           = "--- Exit Sell Fast MA Settings ---";
extern int ExitSellMAFast_Period           = "0";                // MA Period
extern int ExitSellMAFast_Shift            = "0";                // MA Shift
extern int ExitSellMAFast_Method           = "0";                // 0=sma, 1=ema, 2=smma, 3=lwma
extern int ExitSellMAFast_Price            = "0";                // 0=close, 1=open, 2=high, 3=low, 4=median, 5=typical, 6=weighted
// iMA
extern string ExitSellMASlow_iMA           = "--- Exit Sell Slow MA Settings ---";
extern int ExitSellMASlow_Period           = "0";                // MA Period
extern int ExitSellMASlow_Shift            = "0";                // MA Shift
extern int ExitSellMASlow_Method           = "0";                // 0=sma, 1=ema, 2=smma, 3=lwma
extern int ExitSellMASlow_Price            = "0";                // 0=close, 1=open, 2=high, 3=low, 4=median, 5=typical, 6=weighted

// Trade Week Day Filter - @see http://kolier.li/example/script-function-trade-week-day-filter
extern string Trade_Week_Day_Filter           = "--- Trade Week Day ---";
extern bool   TradeWeekDayFilter              = false;              // Whether to use the trade week day filter
extern string TradeDays                       = "2,3,4";            // Choose which days to trade

// Trade Time Filter - @see http://kolier.li/example/script-function-trade-time-filter
extern string Trade_Time_Filter               = "--- Trade Time Filter ---";
extern bool   TradeTimeFilter                 = false;              // Whether to use trade time filter
extern string TradeBegin                      = "07:00";            // Don't trade before this time
extern string TradeEnd                        = "19:00";            // Don't trade after this time
extern bool   EndInTomorrow                   = false;              // Whether TradeEnd is the next day, like you want to trade the period: 18:00~06:00, the 06:00 is the next am

// Order Operation - Open: @see http://kolier.li/example/script-function-order-operation-open
extern string Order_Operation_Open            = "--- Order Operation ---";
extern bool   ECNBroker                       = false;              // Whether using ECN broker
extern int    Magic                           = 222222;             // For identify trades belong to this EA, set to unique on different chart when using at the same time
extern int    Slippage                        = 0;                  // Allow slippage
extern double Lots                            = .01;                // Fixed trade lots without using MoneyManagement
extern double LotsAdd                         = .01;                // Add additional lots to subsequence orders, only work without enable MoneyManagement.
extern bool   MoneyManagement                 = false;              // Whether using Money Management
extern double MM_Risk                         = 0;                  // In Percentage. Risk as equity (for MM_RiskPips), in percent
extern double MM_RiskPips                     = 0;                  // Use this pips value as object for calculating the MM_Risk 
extern int    MM_LotsPrecision                = 0;                  // Precision after zero point.
extern bool   StealthMode                     = false;              // Whether TakeProfit and StopLoss in Stealth Mode, hiding from the broker

extern double StopLoss                        = 100;                // In Pips. Stop Loss, Change to 0 if you don't want to use.
extern double StopLossMinusB1P                = 0;                  // InPips. Reduces Stop Loss Value of Subsequent Orders; 0 to not use.
extern double StopLossMinusB1L                = 0;                  // InPips. Reduces Stop Loss Value of Subsequent Orders; 0 to not use.
extern double StopLossMinusB2P                = 0;                  // InPips. Reduces Stop Loss Value of Subsequent Orders; 0 to not use.
extern double StopLossMinusB2L                = 0;                  // InPips. Reduces Stop Loss Value of Subsequent Orders; 0 to not use.
extern double StopLossMinusS1P                = 0;                  // InPips. Reduces Stop Loss Value of Subsequent Orders; 0 to not use.
extern double StopLossMinusS1L                = 0;                  // InPips. Reduces Stop Loss Value of Subsequent Orders; 0 to not use.
extern double StopLossMinusS2P                = 0;                  // InPips. Reduces Stop Loss Value of Subsequent Orders; 0 to not use.
extern double StopLossMinusS2L                = 0;                  // InPips. Reduces Stop Loss Value of Subsequent Orders; 0 to not use.
extern double TakeProfit                      = 100;                // In Pips. Take Profit, Change to 0 if you don't want to use.
extern double TakeProfitAdd                   = 0;                  // In Pips. Add TP in DD mode.
extern bool   Previous_TakeProfit             = false;              // Limit DD order to use the phase first TakeProfit.

// Order Operation - Break Even: @see http://kolier.li/example/script-function-order-operation-breakeven
extern string Order_Operation_BreakEven       = "--- Break Even ---";
extern bool   UseBreakEven                    = false;              // Whether using BreakEven
extern bool   BE_StealthMode                  = false;              // Whether using BreakEven in Stealth Mode
extern double BreakEvenPips                   = 0;                  // In Pips. BreakEven Pips.
extern bool   BreakEvenWithProfit             = false;              // Different from normal BreakEven, you can make a little in BreakEven, like 1~2 pips
extern double BreakEvenProfit                 = 0;                  // In Pips. BreakEven Pips take profit.

// Order Operation - Trailing Stop: @see http://kolier.li/example/script-function-order-operation-trailing-stop-v2
extern string Order_Operation_Trailing_Stop   = "--- Trailing Stop ---";
extern bool   UseTrailStop1                   = false;              // Whether using TrailStop.
extern bool   TS_StealthMode                  = false;              // Whether using TrailStop in Stealth Mode.
extern double TS_Pips1                        = 0;                  // In Pips. TrailStop Pips.
extern int    TS_StartPips1                   = 0;                  // In Pips. Set to 0 for immediately trigger. Any positive amount as profit achievement.
extern double TS_MovingRange1                 = 0;                  // In Pips. If set larger than 0, the stoploss will only set every range of pips reached.
extern bool   UseTrailStop2                   = false;              // Whether using TrailStop.
extern double TS_Pips2                        = 0;                  // In Pips. TrailStop Pips.
extern int    TS_StartPips2                   = 0;                  // In Pips. Set to 0 for immediately trigger. Any positive amount as profit achievement.
extern double TS_MovingRange2                 = 0;                  // In Pips. If set larger than 0, the stoploss will only set every range of pips reached.
extern bool   UseTrailStop3                   = false;              // Whether using TrailStop.
extern double TS_Pips3                        = 0;                  // In Pips. TrailStop Pips.
extern int    TS_StartPips3                   = 0;                  // In Pips. Set to 0 for immediately trigger. Any positive amount as profit achievement.
extern double TS_MovingRange3                 = 0;                  // In Pips. If set larger than 0, the stoploss will only set every range of pips reached.
//******************************************************************************************************************
//MARTINGALE???!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//******************************************************************************************************************
extern bool    Martingale=False;
extern double  PeakLots=1;//-----------------------highest lots level for Martingale
extern double  PeakLotsMult=1;
extern double  Multiplier=2;
extern double  MinProfit=0;
extern double  MinProfit2=0;
extern double  InitialLots=.01;

extern double  PeakApproach1=.02;
extern double  NewMultiplier1=1.5;
extern int     LoStopLoss=75;
extern int     LoTakeProfit=150;

extern double  PeakApproach2=.12;
extern double  NewMultiplier2=1.5;
extern int     MidStopLoss=75;
extern int     MidTakeProfit=150;

extern double  PeakApproach3=.22;
extern double  NewMultiplier3=1.5;
extern int     HiStopLoss=75;
extern int     HiTakeProfit=150;

extern double  PeakApproach4=.32;
extern double  NewMultiplier4=1.5;
extern int     SupremeStopLoss=75;
extern int     SupremeTakeProfit=150;
extern double  SuperPeakSLMult=1.0;
extern double  SuperPeakTPMult=1.0;//|--was INTENDED to increase lots once PeakLots reached;instead causes profitable loop!
extern int     SuperStopLoss=75;//--------------------SL if lots>=PeakLots value
extern int     SuperTakeProfit=300;//|----------------TP if lots>=PeakLots value

extern bool    MOPP=false;//----CAUSES M'GALE TRADES @ PEAKLOTS TO BE THE OPPOSITE TYPE AS THE FIRST THAT HIT PEAKLOTS;
extern bool    MSAME=false;//----CAUSES M'GALE TRADES @ PEAKLOTS TO BE THE SAME TYPE AS THE FIRST THAT HIT PEAKLOTS;

//+------------------------------------------------------------------+
//| Universal variables                                              |
//+------------------------------------------------------------------+
int days[1], days_size=0;
int bar=0;
int bphase[1], sphase[1], ebphase[1], esphase[1];
datetime btime_phase[1], btime_trade[1], stime_phase[1], stime_trade[1],
         ebtime_phase[1], ebtime_trade[1], estime_phase[1], estime_trade[1];
double pip_step_profit_same, pip_step_profit_opp, pip_step_loss_same, 
       pip_step_loss_opp, pip_step_closed_profit, pip_step_closed_loss,
       pip_step_closed_profit_opp, pip_step_closed_loss_opp, max_spread;

int bmaf_p[1], bmaf_s[1], bmaf_m[1], bmaf_pr[1],
    bmas_p[1], bmas_s[1], bmas_m[1], bmas_pr[1],
    ebmaf_p[1], ebmaf_s[1], ebmaf_m[1], ebmaf_pr[1],
    ebmas_p[1], ebmas_s[1], ebmas_m[1], ebmas_pr[1],
    smaf_p[1], smaf_s[1], smaf_m[1], smaf_pr[1],
    smas_p[1], smas_s[1], smas_m[1], smas_pr[1],
    esmaf_p[1], esmaf_s[1], esmaf_m[1], esmaf_pr[1],
    esmas_p[1], esmas_s[1], esmas_m[1], esmas_pr[1];
double bmin_diff[1], smin_diff[1], ebmin_diff[1], esmin_diff[1];
int bsize=0, ssize=0, ebsize=0, essize=0;
bool flag_gsl = false;
datetime time_over_flag_gsl = 0;

string UUID = "";


//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
//***********************************************************
double maxEquity,minEquity,Balance=0.0,origMultiplier;      
int origStopLoss, origTakeProfit;
//***********************************************************

void Log(string st) {
   Print(st);
}

int init()
  {
//*********************************
   origStopLoss = StopLoss;
   origTakeProfit = TakeProfit;
   origMultiplier = Multiplier;
//*********************************
   
   
   Comment(AdvisorName, "-", AdvisorVersion);
   UUID = AdvisorName + "|" + Magic + "|"+ Symbol() + "|" + Period();
   
   if(Entry_AtBarOpen) {
      bar = 1;
   }
   
   pip_step_profit_same = pips2Points(PipStepProfitSame)*Point;
   pip_step_profit_opp = pips2Points(PipStepProfitOpp)*Point;
   pip_step_loss_same = pips2Points(PipStepLossSame)*Point;
   pip_step_loss_opp = pips2Points(PipStepLossOpp)*Point;

   pip_step_closed_profit = pips2Points(PipStepClosedProfit)*Point;
   pip_step_closed_loss = pips2Points(PipStepClosedLoss)*Point;
   
   pip_step_closed_profit_opp = pips2Points(PipStepClosedProfitOpp)*Point;
   pip_step_closed_loss_opp = pips2Points(PipStepClosedLossOpp)*Point;
   
   
   max_spread = pips2Points(MaxSpread)*Point;
   
   explodeStr2Int(bmaf_p, BuyMAFast_Period);
   explodeStr2Int(bmaf_s, BuyMAFast_Shift);
   explodeStr2Int(bmaf_m, BuyMAFast_Method);
   explodeStr2Int(bmaf_pr, BuyMAFast_Price);
   explodeStr2Int(bmas_p, BuyMASlow_Period);
   explodeStr2Int(bmas_s, BuyMASlow_Shift);
   explodeStr2Int(bmas_m, BuyMASlow_Method);
   explodeStr2Int(bmas_pr, BuyMASlow_Price);
   explodeStr2Points(bmin_diff, BuyMinDiff);
   
   bsize = ArraySize(bmaf_p);
   //Print("bsize: ", bsize);
   ArrayResize(bphase, bsize);
   ArrayResize(btime_phase, bsize);
   ArrayResize(btime_trade, bsize);
   ArrayInitialize(bphase, PHASE_NONE);
   ArrayInitialize(btime_phase, 0);
   ArrayInitialize(btime_trade, 0);
   
   if(ExitBuy_On) {
      explodeStr2Int(ebmaf_p, ExitBuyMAFast_Period);
      explodeStr2Int(ebmaf_s, ExitBuyMAFast_Shift);
      explodeStr2Int(ebmaf_m, ExitBuyMAFast_Method);
      explodeStr2Int(ebmaf_pr, ExitBuyMAFast_Price);
      explodeStr2Int(ebmas_p, ExitBuyMASlow_Period);
      explodeStr2Int(ebmas_s, ExitBuyMASlow_Shift);
      explodeStr2Int(ebmas_m, ExitBuyMASlow_Method);
      explodeStr2Int(ebmas_pr, ExitBuyMASlow_Price);
      explodeStr2Points(ebmin_diff, ExitBuyMinDiff);
   
      ebsize = ArraySize(ebmaf_p);
      //Print("ebsize: ", ebsize);
      ArrayResize(ebphase, ebsize);
      ArrayResize(ebtime_phase, ebsize);
      ArrayResize(ebtime_trade, ebsize);
      ArrayInitialize(ebphase, PHASE_NONE);
      ArrayInitialize(ebtime_phase, 0);
      ArrayInitialize(ebtime_trade, 0);
   }
   
   explodeStr2Int(smaf_p, SellMAFast_Period);
   explodeStr2Int(smaf_s, SellMAFast_Shift);
   explodeStr2Int(smaf_m, SellMAFast_Method);
   explodeStr2Int(smaf_pr, SellMAFast_Price);
   explodeStr2Int(smas_p, SellMASlow_Period);
   explodeStr2Int(smas_s, SellMASlow_Shift);
   explodeStr2Int(smas_m, SellMASlow_Method);
   explodeStr2Int(smas_pr, SellMASlow_Price);
   explodeStr2Points(smin_diff, SellMinDiff);
   
   ssize = ArraySize(smaf_p);
   //Print("ssize: ", ssize);
   ArrayResize(sphase, ssize);
   ArrayResize(stime_phase, ssize);
   ArrayResize(stime_trade, ssize);
   ArrayInitialize(sphase, PHASE_NONE);
   ArrayInitialize(stime_phase, 0);
   ArrayInitialize(stime_trade, 0);
   
   if(ExitSell_On) {
      explodeStr2Int(esmaf_p, ExitSellMAFast_Period);
      explodeStr2Int(esmaf_s, ExitSellMAFast_Shift);
      explodeStr2Int(esmaf_m, ExitSellMAFast_Method);
      explodeStr2Int(esmaf_pr, ExitSellMAFast_Price);
      explodeStr2Int(esmas_p, ExitSellMASlow_Period);
      explodeStr2Int(esmas_s, ExitSellMASlow_Shift);
      explodeStr2Int(esmas_m, ExitSellMASlow_Method);
      explodeStr2Int(esmas_pr, ExitSellMASlow_Price);
      explodeStr2Points(esmin_diff, ExitSellMinDiff);
   
      essize = ArraySize(esmaf_p);
      //Print("essize: ", essize);
      ArrayResize(esphase, essize);
      ArrayResize(estime_phase, essize);
      ArrayResize(estime_trade, essize);
      ArrayInitialize(esphase, PHASE_NONE);
      ArrayInitialize(estime_phase, 0);
      ArrayInitialize(estime_trade, 0);
   }
   
   return(0);
  }

string strTradeType() { return AdvisorName+"_trade_type_"+OrderTicket(); }

//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
   // Clear the un-used Global Variables
   GlobalVariablesDeleteAll("(Test)");
   
   int i;
   for (i = OrdersHistoryTotal() - 1; i >= 0; i--)
   {
      if (OrderSelect(i, SELECT_BY_POS, MODE_HISTORY) == false) continue;
      if (OrderSymbol() != Symbol()) continue;
      if (OrderMagicNumber() != Magic) continue;
    //  Print(OrderComment());
      fxGV(AdvisorName+"_be_able_"+OrderTicket(), DEL);
      fxGV(AdvisorName+"_ts_stoploss_"+OrderTicket(), DEL); 
      fxGV(strTradeType(), DEL);    
   }
   
   
   return(0);
  }

/*
 This varibale indictaes the trade's type:
 0 - undefined
 1 - new trade (RT)
 2 - sub trade
 3 - martiangle
*/
int BUYONLY;
int SELLONLY;

int g_trade_mode = 0;
int SecondWind;
//bool keeponbuy;
//bool keeponsell;
int keepon; 
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
   // BreakEven && TrailStop && StealthMode
   if(scanTrades(TRADETRADE)>0) {
      if(UseBreakEven) breakEven();
      trailStop();
      if(StealthMode) stealthMode();
      

//***********************************************************************
            if(Lots>=PeakLots){
            SecondWind = 1;
            }
//***********************************************************************
   }
   
   if(Martingale)CalculateMartingale();Balance=AccountBalance();//////////////////////////////
   
   resetSignal();
   gslCheck();
   
   double ma_fast[3], ma_slow[3];
   int i, j;
   
   // Buy
   for(i=0; i<bsize; i++) {
      for(j=0; j<3; j++) {
         ma_fast[j] = iMA(NULL, 0, bmaf_p[i], bmaf_s[i], bmaf_m[i], bmaf_pr[i], j);
         ma_slow[j] = iMA(NULL, 0, bmas_p[i], bmas_s[i], bmas_m[i], bmas_pr[i], j);
      }
   
      if(bphase[i]==PHASE_NONE) {
         if(ma_fast[bar]-ma_slow[bar]>=bmin_diff[i]) {
            bphase[i] = PHASE_BUY;
         }
         else if(ma_slow[bar]-ma_fast[bar]>=bmin_diff[i]) {
            bphase[i] = PHASE_SELL;
         }
      }
   
      if(
         bphase[i]==PHASE_SELL && ma_fast[bar]-ma_slow[bar]>=bmin_diff[i]
      ) {
         bphase[i] = PHASE_BUY;
         btime_phase[i] = Time[0];
      }
      if(
         bphase[i]==PHASE_BUY && ma_slow[bar]-ma_fast[bar]>=bmin_diff[i]
      ) {
         bphase[i] = PHASE_SELL;
         btime_phase[i] = Time[0];
      }
   }
   // Exit Buy
   if(ExitBuy_On) {
      for(i=0; i<ebsize; i++) {
         for(j=0; j<3; j++) {
            ma_fast[j] = iMA(NULL, 0, ebmaf_p[i], ebmaf_s[i], ebmaf_m[i], ebmaf_pr[i], j);
            ma_slow[j] = iMA(NULL, 0, ebmas_p[i], ebmas_s[i], ebmas_m[i], ebmas_pr[i], j);
         }
   
         if(ebphase[i]==PHASE_NONE) {
            if(ma_fast[bar]-ma_slow[bar]>=ebmin_diff[i]) {
               ebphase[i] = PHASE_BUY;
            }
            else if(ma_slow[bar]-ma_fast[bar]>=ebmin_diff[i]) {
               ebphase[i] = PHASE_SELL;
            }
         }
   
         if(
            ebphase[i]==PHASE_SELL && ma_fast[bar]-ma_slow[bar]>=ebmin_diff[i]
         ) {
            ebphase[i] = PHASE_BUY;
            ebtime_phase[i] = Time[0];
         }
         if(
            ebphase[i]==PHASE_BUY && ma_slow[bar]-ma_fast[bar]>=ebmin_diff[i]
         ) {
            ebphase[i] = PHASE_SELL;
            ebtime_phase[i] = Time[0];
         }
      }
   }
   // Sell
   for(i=0; i<ssize; i++) {
      for(j=0; j<3; j++) {
         ma_fast[j] = iMA(NULL, 0, smaf_p[i], smaf_s[i], smaf_m[i], smaf_pr[i], j);
         ma_slow[j] = iMA(NULL, 0, smas_p[i], smas_s[i], smas_m[i], smas_pr[i], j);
      }
   
      if(sphase[i]==PHASE_NONE) {
         if(ma_fast[bar]-ma_slow[bar]>=smin_diff[i]) {
            sphase[i] = PHASE_BUY;
         }
         else if(ma_slow[bar]-ma_fast[bar]>=smin_diff[i]) {
            sphase[i] = PHASE_SELL;
         }
      }
   
      if(
         sphase[i]==PHASE_SELL && ma_fast[bar]-ma_slow[bar]>=smin_diff[i]
      ) {
         sphase[i] = PHASE_BUY;
         stime_phase[i] = Time[0];
      }
      if(
         sphase[i]==PHASE_BUY && ma_slow[bar]-ma_fast[bar]>=smin_diff[i]
      ) {
         sphase[i] = PHASE_SELL;
         stime_phase[i] = Time[0];
      }
   }
   // Exit Sell
   if(ExitSell_On) {
      for(i=0; i<essize; i++) {
         for(j=0; j<3; j++) {
            ma_fast[j] = iMA(NULL, 0, esmaf_p[i], esmaf_s[i], esmaf_m[i], esmaf_pr[i], j);
            ma_slow[j] = iMA(NULL, 0, esmas_p[i], esmas_s[i], esmas_m[i], esmas_pr[i], j);
         }
   
         if(esphase[i]==PHASE_NONE) {
            if(ma_fast[bar]-ma_slow[bar]>=esmin_diff[i]) {
               esphase[i] = PHASE_BUY;
            }
            else if(ma_slow[bar]-ma_fast[bar]>=esmin_diff[i]) {
               esphase[i] = PHASE_SELL;
            }
         }
   
         if(
            esphase[i]==PHASE_SELL && ma_fast[bar]-ma_slow[bar]>=esmin_diff[i]
         ) {
            esphase[i] = PHASE_BUY;
            estime_phase[i] = Time[0];
         }
         if(
            esphase[i]==PHASE_BUY && ma_slow[bar]-ma_fast[bar]>=esmin_diff[i]
         ) {
            esphase[i] = PHASE_SELL;
            estime_phase[i] = Time[0];
         }
      }
   }
   
   int ticket;
   // Trade Time Filter
   if((!TradeTimeFilter || (TradeTimeFilter && tradeTime()))
      && (!TradeWeekDayFilter || (TradeWeekDayFilter && tradeWeekDay()))) {
      if(
         (Ask-Bid<=max_spread)
      ) {
         // Buy
         for(i=0; i<bsize; i++) {
            if(
               bphase[i]==PHASE_BUY && btime_phase[i]>btime_trade[i] && pipStepCheck(OP_BUY) && pipStepCheck2(OP_BUY)
               && pipStepCheck3(OP_BUY) /*&& Alternate (OP_BUY)*/ && Sameternate (OP_BUY) && Oppternate (OP_BUY) && BOSO (OP_BUY) 
               && ( MaxOrders==0 || (MaxOrders>0 && scanTrades(TRADETRADE)<MaxOrders) )
            ) {
               //Print(i);
               ticket = tradeBuy();
               if(ticket > 0) {
                  btime_trade[i] = btime_phase[i];
               }
            }
            else {
               btime_phase[i] = 0;
            }
         }
         // Sell
         for(i=0; i<ssize; i++) {
            if(
               sphase[i]==PHASE_SELL && stime_phase[i]>stime_trade[i] && pipStepCheck(OP_SELL) && pipStepCheck2(OP_SELL)
               && pipStepCheck3(OP_SELL) /*&& Alternate (OP_SELL)*/ && Sameternate (OP_SELL) && Oppternate (OP_SELL) && BOSO (OP_SELL)
               && ( MaxOrders==0 || (MaxOrders>0 && scanTrades(TRADETRADE)<MaxOrders) )
            ) {
               ticket = tradeSell();
               if(ticket > 0) {
                  stime_trade[i] = stime_phase[i];
               }
            }
            else {
               stime_phase[i] = 0;
            }
         }
      }
   }
   
   if(ExitBuy_On) {
      int buy_time = 0;
      for(i=0; i<bsize; i++) {
         if(buy_time < btime_trade[i]) {
            buy_time = btime_trade[i];
         }
      }
      for(i=0; i<ebsize; i++) {
         if(ebphase[i]==PHASE_SELL && ebtime_phase[i]>buy_time //&& pipStepCheck(OP_SELL)
         ) {
            closeTradesBuy();
         }
      }
   }
   if(ExitSell_On) {
      int sell_time = 0;
      for(i=0; i<ssize; i++) {
         if(sell_time < stime_trade[i]) {
            sell_time = stime_trade[i];
         }
      }
      for(i=0; i<essize; i++) {
         if(ebphase[i]==PHASE_BUY && estime_phase[i]>sell_time //&& pipStepCheck(OP_BUY)
         ) {
            closeTradesSell();
         }
      }
   }
   
   return(0);
  }
  
//+------------------------------------------------------------------+
//| Reset entry signals when the new closed order make the currently opening amount be (MaxOrder - 1)
//+------------------------------------------------------------------+
void resetSignal()
  {
   if(MaxOrders == 0) {
      return;
   }
   static int orders_opening = 0;
   int orders_opening_tmp = scanTrades(TRADETRADE);
   if (orders_opening > orders_opening_tmp && orders_opening_tmp == (MaxOrders - 1)) {
      int i;
      for(i=0; i<bsize; i++) {
         btime_trade[i] = btime_phase[i];
      }
      // Sell
      for(i=0; i<ssize; i++) {
         stime_trade[i] = stime_phase[i];
      }
      // Reset GSL.
      //flag_gsl = false;
   }
   
   orders_opening = orders_opening_tmp;
  }
  
//+------------------------------------------------------------------+
//| Ghost stoploss                                                   |
//+------------------------------------------------------------------+
void gslCheck()
{
    if (!Ghost_Stoploss) return;

    int ticket_phase;
    static double price = 0;
    static datetime time = 0;
    
    // Check in state.
    ticket_phase = findPhaseOrder(OP_BUY);
    if (ticket_phase > 0)
    {
        OrderSelect(ticket_phase, SELECT_BY_TICKET);
        price = OrderOpenPrice();
        time = OrderOpenTime();
    }
    else
    {
        ticket_phase = findPhaseOrder(OP_SELL);
        if (ticket_phase > 0)
        {
            OrderSelect(ticket_phase, SELECT_BY_TICKET);
            price = OrderOpenPrice();
            time = OrderOpenTime();
        }
        else
        {
            return;
        }
    }

    // Controller.
    if (ticket_phase > 0)
    {
        flag_gsl = true;
    }
    else
    {
        flag_gsl = false;
    }

    // Operate on state.
    if (flag_gsl)
    {
        gslWork(time);
    }
    
    // Check out state.
    if (flag_gsl)
    {
        double diff = pips2Points(MathMin(Slippage, 3)) * Point;
        if (MathAbs(Close[0] - price) <= diff)
        {
            flag_gsl = false;
            time_over_flag_gsl = TimeCurrent();
        }
    }
}

void gslWork(datetime time)
{
    if (!flag_gsl || scanTrades(TRADETRADE) == 0) return;

    for (int i = 0; i < OrdersTotal(); i++)
    {
        if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES) == false) continue;
        // Remove all the DD stoploss.
        if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic && OrderComment() == "DD" && OrderStopLoss() > 0 && OrderOpenTime() > time)
        {
            OrderModify(OrderTicket(), OrderOpenPrice(), 0, OrderTakeProfit(), 0, CLR_NONE);
        }
    }
}

/**
 * Get the ghost #1 order.
 */
int ddMapGhost()
{
    int order_ghost = -1;
    string name_ghost = UUID + "|Order|Ghost";
    order_ghost = fxGV(name_ghost);
    int ticket = -1;
    int i;
    datetime time_order = TimeCurrent(), time_order_ghost = 0;
    
    // Assign the logged order_ghost.
    if (order_ghost > 0)
    {
        // @debug
        //Print(order_ghost);
        OrderSelect(order_ghost, SELECT_BY_TICKET);
        time_order = OrderOpenTime();
        // If order_ghost is closed.
        if (OrderCloseTime() > 0)
        {
            time_order_ghost = OrderOpenTime();
        }
        // Reset the order_ghost.
        if (
            OrderProfit() > 0
            || time_over_flag_gsl > OrderOpenTime()
        )
        {
            // @debug
            //Print(OrderTicket());
            order_ghost = -1;
            time_order = TimeCurrent();
            time_order_ghost = 0;
        }
    }

    for (i = 0; i < OrdersTotal(); i++)
    {
        if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES) == false) continue;
        if (OrderSymbol() != Symbol() || OrderMagicNumber() != Magic) continue;
        // time_over_flag_gsl guarantee the order is older than previous ghost phase.
        if (time_order >= OrderOpenTime() && OrderOpenTime() > time_over_flag_gsl && OrderProfit() < 0)
        {
            time_order = OrderOpenTime();
            ticket = OrderTicket();
        }
    }
    // Need to have order after the ghost order.
    int counter = 0;
    if (ticket > 0)
    {
        for (i = 0; i < OrdersTotal(); i++)
        {
            if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES) == false) continue;
            if (OrderSymbol() != Symbol() || OrderMagicNumber() != Magic) continue;
            if (OrderOpenTime() > time_order)
            {
                counter++;
            }
        }
        if (counter < 1)
        {
            ticket = -1;
        }
    }
    
    // @debug
    //if (ticket > 0) Print(ticket);

    if (ticket > 0)
    {
        if (ticket == order_ghost)
        {
            // #1 still open.
        }
        else if (ticket != order_ghost)
        {
            // New #1.
            order_ghost = ticket;
        }
    }
    else if (ticket < 0)
    {
        // If the ghost get closed, check if that ghost phase is finished.
        // If not yet finished, fetch the logged order_ghost.
        if (order_ghost > 0 && time_order_ghost > 0 && time_order_ghost > time_over_flag_gsl)
        {
            ticket = order_ghost;
        }
    }
    
    // @debug
    //if (ticket > 0) Print(ticket);

    fxGV(name_ghost, SET, order_ghost);
    
    return(ticket);
}
 ////////////////////////////////////////////////////////////////////////////////////////////***NEW PIPSTEP*** 
/////PIPSTEP AS OF 11-15-15 WITH ANTI-BACKSTEP (ABS) (PSCL,PSCP ONLY):
//+------------------------------------------------------------------+
//| Check the pip step condition. @http://kolier.li                  |
//+------------------------------------------------------------------+
bool pipStepCheck(int op_type)
  {
   bool cond = true;
   int i, ticket, ticket_phase = -1;
   double price_open;
   double price_close;//////////////////////////////
   
   if(op_type == OP_BUY) {
         
         ticket_phase = findPhaseOrder(OP_BUY);
         if(ticket_phase >= 0) {
            
            OrderSelect(ticket_phase, SELECT_BY_TICKET);
            price_open = OrderOpenPrice();

         }
////////////////////////////////////////////////////////////////////////////////////////////////////         
         if(OrderSymbol()==Symbol() && OrderMagicNumber()==Magic && OrderType()==OP_BUY) {   //(Order that is there...)
       if(
         (pip_step_loss_same > 0 && OrderProfit()< 0 && (price_open-Ask >=-1//////////////////*
         && (price_open-Ask < pip_step_loss_same)))
         )
         {
         cond = false;
         }
       if(  
         (pip_step_profit_same > 0 && OrderProfit()> 0 && (Ask-price_open >=-1////////////////*
         && (Ask-price_open < pip_step_profit_same)))
         ) 
         {
         cond = false;
         }
      }               //***'ACTIVATE' FOR PSCP/PSCL
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////      
      ticket = fxOrderFindLatestClosed(Symbol(), Magic, op_type);//SAME
      if(ticket > -1)                                            //CC 10-28-15
      {                                                          //CC 10-28-15
      OrderSelect(ticket, SELECT_BY_TICKET);                     //SAME
      if(ticket_phase < 0) {                                     //NEW
         price_open = OrderOpenPrice();                          //NEW
         price_close = OrderClosePrice();//////////////////////////////
      }                                                          //NEW
       if(
         (pip_step_closed_loss > 0 && OrderProfit()< 0 && 
         (price_open-Ask > 0 && (price_open-Ask < price_open-price_close 
         || (price_close-Ask > 0 && (price_close-Ask < pip_step_closed_loss)))))
         )
         {
         cond = false;
         }
       if(  
         (pip_step_closed_profit > 0 && OrderProfit()> 0 && 
         (Ask-price_open > 0 && (Ask - price_open < price_close-price_open
         || (Ask-price_close > 0 && (Ask-price_close < pip_step_closed_profit)))))
         ) 
         {
         cond = false;
         }                        //***'ACTIVATE' FOR PSCP/PSCL
      }//CC 10-28-15              //***'ACTIVATE' FOR PSCP/PSCL
   }
   else if(op_type == OP_SELL) {

         ticket_phase = findPhaseOrder(OP_SELL);
         if(ticket_phase >= 0) {
            
            OrderSelect(ticket_phase, SELECT_BY_TICKET);
            price_open = OrderOpenPrice();

         }
////////////////////////////////////////////////////////////////////////////////////////////////////         
         if(OrderSymbol()==Symbol() && OrderMagicNumber()==Magic && OrderType()==OP_SELL) {   //(Order that is there...)***
       if(
         (pip_step_loss_same > 0 && OrderProfit()< 0 && (Bid - price_open >=-1//////////////////*
         && (Bid - price_open < pip_step_loss_same)))
         )
         {
         cond = false;
         }
       if(  
         (pip_step_profit_same > 0 && OrderProfit()> 0 && (price_open - Bid >=-1////////////////*
         && (price_open-Bid < pip_step_profit_same)))
         )
         {
         cond = false;
         }
         //}
      }               //***'ACTIVATE' FOR PSCP/PSCL
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////      
      ticket = fxOrderFindLatestClosed(Symbol(), Magic, op_type);
      if(ticket > -1)                                            //CC 10-28-15
      {                                                          //CC 10-28-15
      OrderSelect(ticket, SELECT_BY_TICKET);
      if(ticket_phase < 0) {
         price_open = OrderOpenPrice();
         price_close = OrderClosePrice();//////////////////////////////
      }
       if(
         (pip_step_closed_loss > 0 && OrderProfit()< 0 && 
         (Bid-price_open > 0 && (Bid - price_open < price_close-price_open
         || (Bid - price_close > 0 && (Bid - price_close < pip_step_closed_loss)))))
         )
         {
         cond = false;
         }
       if(  
         (pip_step_closed_profit > 0 && OrderProfit()> 0 && 
         (price_open-Bid > 0 && (price_open - Bid < price_open-price_close
         || (price_close - Bid > 0 && (price_close-Bid < pip_step_closed_profit)))))
         )
         {
         cond = false;
         }               //***'ACTIVATE' FOR PSCP/PSCL
      }//CC 10-28-15     //***'ACTIVATE' FOR PSCP/PSCL
   }
   
   return(cond);
  }

////////////////////////////////////////////////////////////////////////////////////////////***NEW PIPSTEP(2)***

bool pipStepCheck2(int op_type)
  {
   bool cond = true;
   int i, ticket, ticket_phase = -1;
   double price_open;
   double price_close;//////////////////////////////
   
         if(op_type == OP_BUY) {
         
         ticket_phase = findPhaseOrder(OP_SELL);
         if(ticket_phase >= 0) {
            
            OrderSelect(ticket_phase, SELECT_BY_TICKET);
            price_open = OrderOpenPrice();
       }
       if(OrderSymbol()==Symbol() && OrderMagicNumber()==Magic && OrderType()==OP_SELL) {   //(OPP Order that is there...)
       if(
         (pip_step_loss_opp > 0 && OrderProfit()< 0 && (Bid - price_open >=-1//////////////////*
         && (Bid - price_open < pip_step_loss_opp)))
         )
         {
         cond = false;
         }
       if(  
         (pip_step_profit_opp > 0 && OrderProfit()> 0 && (price_open - Bid >=-1////////////////*
         && (price_open-Bid < pip_step_profit_opp)))
         )
         {
         cond = false;
         }
      }
   }

   else if(op_type == OP_SELL) {

         ticket_phase = findPhaseOrder(OP_BUY);
         if(ticket_phase >= 0) {
            
            OrderSelect(ticket_phase, SELECT_BY_TICKET);
            price_open = OrderOpenPrice();
         }
       if(OrderSymbol()==Symbol() && OrderMagicNumber()==Magic && OrderType()==OP_BUY) {   //(OPP Order that is there...)
       if(
         (pip_step_loss_opp > 0 && OrderProfit()< 0 && (price_open-Ask >=-1//////////////////*
         && (price_open-Ask < pip_step_loss_opp)))
         )
         {
         cond = false;
         }
       if(  
         (pip_step_profit_opp > 0 && OrderProfit()> 0 && (Ask-price_open >=-1////////////////*
         && (Ask-price_open < pip_step_profit_opp)))
         ) 
         {
         cond = false;
         }
       }
    }
   return(cond);
  }
////////////////////////////////////////////////////////////////////////////////////////////////////////////START*PIPSTEP3***
bool pipStepCheck3(int op_type)
  {
   bool cond = true;
   int i, ticket, ticket_phase = -1;
   double price_open;
   double price_close;//////////////////////////////
   
   if(op_type == OP_BUY) {
         

      ticket = fxOrderFindLatestClosed(Symbol(), Magic, OP_SELL);//SAME
      if(ticket > -1)                                            //CC 10-28-15
      {                                                          //CC 10-28-15
      OrderSelect(ticket, SELECT_BY_TICKET);                     //SAME
      if(ticket_phase < 0) {                                     //NEW
         price_open = OrderOpenPrice();                          //NEW
         price_close = OrderClosePrice();//////////////////////////////
      }                                                          //NEW
       if(
         (pip_step_closed_loss_opp > 0 && OrderProfit()< 0 && 
         (Bid-price_open > 0 && (Bid - price_open < price_close-price_open
         || (Bid - price_close > 0 && (Bid - price_close < pip_step_closed_loss_opp)))))
         )
         {
         cond = false;
         }
       if(  
         (pip_step_closed_profit_opp > 0 && OrderProfit()> 0 && 
         (price_open-Bid > 0 && (price_open - Bid < price_open-price_close
         || (price_close - Bid > 0 && (price_close-Bid < pip_step_closed_profit_opp)))))
         )
         {
         cond = false;
         }                        //***'ACTIVATE' FOR PSCP/PSCL
      }//CC 10-28-15              //***'ACTIVATE' FOR PSCP/PSCL
   }
   else if(op_type == OP_SELL) {

      ticket = fxOrderFindLatestClosed(Symbol(), Magic, OP_BUY);
      if(ticket > -1)                                            //CC 10-28-15
      {                                                          //CC 10-28-15
      OrderSelect(ticket, SELECT_BY_TICKET);
      if(ticket_phase < 0) {
         price_open = OrderOpenPrice();
         price_close = OrderClosePrice();//////////////////////////////
      }
       if(
         (pip_step_closed_loss_opp > 0 && OrderProfit()< 0 && 
         (price_open-Ask > 0 && (price_open-Ask < price_open-price_close 
         || (price_close-Ask > 0 && (price_close-Ask < pip_step_closed_loss_opp)))))
         )
         {
         cond = false;
         }
       if(  
         (pip_step_closed_profit_opp > 0 && OrderProfit()> 0 && 
         (Ask-price_open > 0 && (Ask - price_open < price_close-price_open
         || (Ask-price_close > 0 && (Ask-price_close < pip_step_closed_profit_opp)))))
         ) 
         {
         cond = false;
         }               //***'ACTIVATE' FOR PSCP/PSCL
      }//CC 10-28-15     //***'ACTIVATE' FOR PSCP/PSCL
   }
   
   return(cond);
  }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//RT = REFERENCE TRADE (THE VERY FIRST TRADE IN A SERIES OR GROUP-TO-BE); TRADES OCCURRING DURING THE FIRST ARE 'SUBSEQUENTS';
//HERE ARE THE 2 FUNCTIONS IN QUESTION---OPPTERNATE WORKS GREAT, BUT SAMETERNATE YIELDS NO TRADES AT ALL, 
//OR YIELDS EITHER ALL BUYS OR ALL SELLS IF RT_type IS MANUALLY SET TO 0 OR 1, WHETHER INTERNALLY OR EXTERNALLY---
//RT_type NEEDS TO SWITCH TO THE RT's ACTUAL TYPE!: IN SAMETERNATE, IT IS SET TO SELL-ONLY (1) FOR ILLUSTRATIVE PURPOSES.
//WHAT IS NEEDED IS TO MAKE SAMETERNATE ALLOW ONLY THE SAME TRADE TYPE AS THE RT TO OCCUR WHILE THE RT IS OPEN. THANK YOU. 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Oppternate (int op_type)
{
    if(!Oppternate) return true;
   
    datetime time_latest_tp = 0, time_phase = EMPTY_VALUE;
    int ticket = -1;
    int i;
    int RT_type = -1;

    for (i = 0; i < OrdersTotal(); i++)
    {
        OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
        if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic/* && OrderComment() == "New"*/)
        {
            if (OrderOpenTime() > time_latest_tp 
            && OrderOpenTime() < time_phase 
            && OrderComment() == "New")
            {
                time_phase = OrderOpenTime();
                ticket = OrderTicket();
                RT_type = OrderType();
                Print("THIS IS THE RT      ",ticket,"  Order Type   ",OrderType(),"  op_type ",op_type);
            }
        }
    }
            if(Oppternate)
            {
            if 
            (RT_type != op_type)
            {
            return true;
            }
            else
            {
            return false;
            }
      }
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool Sameternate (int op_type)
{
    if(!Sameternate) return true;
   
    datetime time_latest_tp = 0, time_phase = EMPTY_VALUE;
    int ticket = -1;
    int i;
    int RT_type = 1;

    for (i = 0; i < OrdersTotal(); i++)
    {
        OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
        if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic/* && OrderComment() == "New"*/)
        {
            if (OrderOpenTime() > time_latest_tp 
            && OrderOpenTime() < time_phase 
            && OrderComment() == "New")
            {
                time_phase = OrderOpenTime();
                ticket = OrderTicket();
                RT_type = OrderType();
                Print("THIS IS THE RT      ",ticket,"  Order Type   ",OrderType(),"  op_type ",op_type);
            }
        }
    }
            if(Sameternate)
            {
            if 
            (RT_type == op_type)
            {
            return true;
            }
            else
            {
            return false;
            }
      }
}
     


///////////////////////////////////////////////////////////////////////////////////////////////////////////
//END OF SECTION IN-QUESTION.                                                                           ///
///////////////////////////////////////////////////////////////////////////////////////////////////////////

/*  
void FindLastTrades(int& ticket_open, int& dir_open, int& ticket_hist, int& dir_hist) {
   ticket_open = -1; 
   ticket_hist = -1;
   datetime optime1 = 0, optime2 = 0;   
   int i = 0;
   
   for(i=OrdersTotal()-1; i>=0; i--) {
      if(!OrderSelect(i,SELECT_BY_POS)) continue;
      if(OrderSymbol()==Symbol() && (Magic == 0 || OrderMagicNumber() == Magic)) 
      if(OrderOpenTime()>optime1) {
         ticket_open = OrderTicket();
         optime1 = OrderOpenTime();
         dir_open = OrderType();
         //break;
      }
   }
   
   for(i=OrdersHistoryTotal()-1; i>=0; i--) {
      if(!OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)) continue;
      if(OrderSymbol()==Symbol() && (Magic == 0 || OrderMagicNumber() == Magic)) 
      if(OrderOpenTime()>optime2){
         ticket_hist = OrderTicket();
         optime2 = OrderOpenTime();
         dir_hist = OrderType();
        // break;
      }
   }
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////END*PIPSTEP3***
//Alternate!!!

bool Alternate (int op_type) {
         
         int ticket1 = -1, ticket2 = -1;         
         int dir1, dir2, dir;         
         
         FindLastTrades(ticket1, dir1, ticket2, dir2);
         
         
         if(ticket1<0 && ticket2<0) return true;
         if(ticket1<ticket2) {
            dir = dir2;
         } else {
            dir = dir1;
         } 
         Log("ticvket1="+ticket1+",ticket2="+ticket2);                     
         if(Alternate) {
            if((op_type==OP_SELL && dir==OP_BUY)||(op_type==OP_BUY && dir==OP_SELL)) {
               Log("Alternate1");
               return true;
            }
         } else if(!Alternate) {
            Log("Alternate2");
            return true;
         }
         return false;
}
         

/////////////////////////////////////////////////////////////////////////////////////////////////////END Alternate***

//Sameternate////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Sameternate (int op_type) {
   
   if(!Sameternate) {
      return true;
   }
   
   int ticket1 = -1, ticket2 = -1;         
   int dir1, dir2, dir = -1;         
         
   FindLastTrades(ticket1, dir1, ticket2, dir2);
   if(ticket1<0 && ticket2<0) {
      //Print("**** " + op_type);     
      return true;
   }
         
   if(Sameternate) {
      if((ticket1>0 && op_type==dir1)||ticket1<0) return true;      
      //if(ticket2>0 && op_type==dir2) return true;   
   } 
  // Print(op_type+","+ticket1);
   return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////END Samternate***

//Oppternate////////////////////////////////////////////////////////////////////////////////////////////////////////

bool Oppternate (int op_type) {
   
   if(!Oppternate) return true;
   
   datetime optime1, optime2;
   int dir1=-1;
   //find the earliest trade
   for(int i=OrdersTotal()-1; i>=0; i--) {
      if(!OrderSelect(i,SELECT_BY_POS)) continue;
      if(OrderSymbol()==Symbol() && (Magic == 0 || OrderMagicNumber() == Magic)) {
         if(OrderOpenTime()<optime1 || optime1==0) {
            //ticket1 = OrderTicket();
            optime1 = OrderOpenTime();
            dir1 = OrderType();              
         }
      }
   }
   
   if(optime1==0) return true;
   optime2= optime1;
   int dir2 = -1;
   bool found = true;
   while(found) {
      found = false;
      for(i=OrdersHistoryTotal()-1; i>=0; i--) {
         if(!OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)) continue;
         if(OrderSymbol()==Symbol() && (Magic == 0 || OrderMagicNumber() == Magic)) {
            if(OrderOpenTime()<optime2 && OrderCloseTime()>optime2) {
               //ticket2 = OrderTicket();
               optime2 = OrderOpenTime();
               dir2 = OrderType();    
               found = true;
               break;          
            }
         }
      }
   }
   
   if(dir2==-1) dir2=dir1;
   
   return(dir2!=op_type);
}
*/     
//////////////////////////////////////////////////////////////////////////////////////////////////////END Oppternate***
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
 * Find the oldest opened order on the same phase.
 * 1. Get the time of last takeprofit order.
 * 2. Check if new order open after (its) opening or closed with loss.
 */
int findPhaseOrder(int op_type)
{
    if (DD_MapGhost)
    {
        return(ddMapGhost());
    } 

    datetime time_latest_tp = 0, time_phase = EMPTY_VALUE;
    int ticket = -1;
    int i;/*
    for (i = 0; i < OrdersHistoryTotal(); i++)
    {
        OrderSelect(i, SELECT_BY_POS, MODE_HISTORY);
        if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic && OrderType()==op_type)
        {
            if (OrderProfit() <= 0 && OrderCloseTime() > time_latest_tp)   //WAS if (OrderProfit() >= 0 Major change 3-9-17
            {
                time_latest_tp = OrderCloseTime();
                //ticket = OrderTicket();
                //Print("GPO 1 XXXXX   ");
            }
        }
    }
    
    if (time_latest_tp == 0)
    {
        return(-2);
    }

    for (i = 0; i < OrdersHistoryTotal(); i++)
    {
        OrderSelect(i, SELECT_BY_POS, MODE_HISTORY);
        if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic && OrderType()==op_type)
        {
            if (OrderProfit() < 0 
            && OrderOpenTime() > time_latest_tp 
            && OrderOpenTime() < time_phase 
            && OrderComment() == "New")
            {
                time_phase = OrderOpenTime();
                ticket = OrderTicket();
                //Print("GPO 2 XXXXXXXXXX------------",ticket,"     ",OrderType());
            }
        }
    }*/
    for (i = 0; i < OrdersTotal(); i++)
    {
        OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
        if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic && OrderType()==op_type)
        {
            if (/*OrderProfit() < 0 && */                               //MAJOR CHANGE 1-6-17
               OrderOpenTime() > time_latest_tp 
            && OrderOpenTime() < time_phase 
            && OrderComment() == "New")
            {
                time_phase = OrderOpenTime();
                ticket = OrderTicket();
                //Print("GPO 3 XXXXXXXXXXXXXXX->->->->->",ticket,"     ",OrderType());
            }
        }
    }
    
    return(ticket);
}

/**
 * Map the DD order to 
 */
void ddMapRT(int type, int &ticket)
{
    int order_rt[2];
    string name_rt[2];
    string prefix = UUID + "|Order|RT";
    name_rt[0] = prefix + "|Buy";
    name_rt[1] = prefix + "|Sell";
    order_rt[0] = fxGV(name_rt[0]);
    order_rt[1] = fxGV(name_rt[1]);
    
    if (ticket == -1 && order_rt[type] > 0)
    {
        ticket = order_rt[type];
        return;
    }
    
    if (type == OP_BUY)
    {
        // If already store a "New" order.
        // Check if newer to update.
        if (order_rt[type] > 0)
        {
            if (fxOrderVOpenPrice(ticket) > fxOrderVOpenPrice(order_rt[type]))
            {
                order_rt[type] = ticket;
            }
            else
            {
                ticket = order_rt[type];
            }
        }
        // If store nothing, just use it.
        else
        {
            order_rt[type] = ticket;
        }
    }
    else if (type == OP_SELL)
    {
        if (order_rt[type] > 0)
        {
            if (fxOrderVOpenPrice(ticket) < fxOrderVOpenPrice(order_rt[type]))
            {
                order_rt[type] = ticket;
            }
            else
            {
                ticket = order_rt[type];
            }
        }
        else
        {
            order_rt[type] = ticket;
        }
    }

    fxGV(name_rt[0], SET, order_rt[0]);
    fxGV(name_rt[1], SET, order_rt[1]);
}
  
//+------------------------------------------------------------------+
//| Whether today is the trading day in the week @http://kolier.li   |
//|   @see http://kolier.li/example/script-function-trade-week-day-filter
//| explodeStr2Int()
//|   @see http://kolier.li/example/stript-function-string-explode
//+------------------------------------------------------------------+
bool tradeWeekDay()
  {
   if(days_size==0) {
      explodeStr2Int(days, TradeDays);
      days_size = ArraySize(days);
   } 
   for(int i=0; i<days_size; i++) {
      if(DayOfWeek()==days[i]) {
         return(true);
      }
   }
   return(false);
  }  
  
  
//+------------------------------------------------------------------+
//| Explode String to Integer @http://kolier.li                      |
//+------------------------------------------------------------------+  
void explodeStr2Int(int& arr[], string text_arr, string separator=",")
  {
    int    start = 0,
           length,
           array_size;
    bool   eos = false;
    
    ArrayResize(arr, 0);
    
    while(eos==false) {
      length = StringFind(text_arr,separator,start)-start;
      if(length < 0) {
         // Avoid empty string
         if(StringLen(text_arr)==0) {
            return;
         }
         eos = true;
         length = StringLen(text_arr)-start;
      }
      
      array_size = ArraySize(arr);
      ArrayResize(arr, array_size+1);
      arr[array_size] = StrToInteger(StringSubstr(text_arr, start, length));
      start += length+1;
    }
  }  

//+------------------------------------------------------------------+
//| Trade Time Filter @http://kolier.li                              |
//|   @see http://kolier.li/example/script-function-trade-time-filter|
//+------------------------------------------------------------------+
bool tradeTime()
  {
   if(TradeTimeFilter) {
      datetime time_now, time_begin, time_end, time_end_today;
      time_now = TimeCurrent();
      time_begin = StrToTime(TradeBegin);
      if(!EndInTomorrow) {
         time_end = StrToTime(TradeEnd);
      }
      else if(EndInTomorrow) {
         time_end = StrToTime(TradeEnd) + 86400;
         time_end_today = StrToTime(TradeEnd);
      }
      if(!EndInTomorrow && (time_now<time_begin || time_now>time_end)) {
         return(false);
      }
      else if(EndInTomorrow && time_now>time_end_today && time_now<time_begin) {
         return(false);
      }
   }
   return(true);  
  }
  
//+------------------------------------------------------------------+
//| Trade Buy @http://kolier.li @v2.0                                |
//| @import                                                          |
//|   bool ECNBroker                                                 |
//|   double TakeProfit                                              |
//|   double StopLoss                                                |
//|   double Lots                                                    |
//+------------------------------------------------------------------+
int tradeBuy(double tp=0, double sl=0, double lots=0)
  {
   double price_now, price_profit, price_stop;
   double take_profit = pips2Points(TakeProfit)*Point;
   if(tp>0) take_profit = tp; 
   double stop_loss = pips2Points(StopLoss)*Point;
   if(sl>0) stop_loss = sl;
   int ticket;
   price_now = Ask;
   if(take_profit > 0) {
      price_profit = price_now + take_profit;
   }
   else {
      price_profit = 0;
   }
   if(stop_loss > 0) {
      price_stop = price_now - stop_loss;
   }
   else {
      price_stop = 0;
   }

   if(MoneyManagement && lots==0) {
      lots = mmLots();
   } 
 
   if(lots==0) lots = Lots;
   
   string cmt = "New";  
   
   //Since the martiangle pramaters may be set in a different tick we don't change it if it was 3.
   
   if(3!=g_trade_mode) {
      g_trade_mode = 1;
      //Print("g3");
   }
   int ticket_phase = findPhaseOrder(OP_BUY);
   double phase_price_op, phase_price_tp, phase_profit, phase_lots;
   int phase_time_open;
   int j;
   int SLMB1P=StopLossMinusB1P;
   int SLMB1L=StopLossMinusB1L;
   if(ticket_phase > 0) {
      
      OrderSelect(ticket_phase, SELECT_BY_TICKET);
      phase_price_op = OrderOpenPrice();
      phase_price_tp = OrderTakeProfit();
      phase_profit = OrderProfit();
      phase_lots = OrderLots();
      phase_time_open = OrderOpenTime();
            
      if(phase_profit > 0) {
         cmt = "BUY(duringbuyin(+)) RT " + ticket_phase;
         /*if(!MoneyManagement) {
            lots = phase_lots + LotsAdd;
         }*/
	      
            if(StopLossMinusB1P) {
               if(StopLoss-StopLossMinusB1P<6) 
                  {
                     SLMB1P=6;
                  }
               price_stop = price_stop + pips2Points(SLMB1P)*Point;
               Print("B1P     ",price_now - price_stop);
            }                     
            //if(Previous_TakeProfit) {
            //price_profit = phase_price_tp;
            //}
            //else {
            price_profit = price_profit + pips2Points(TakeProfitAdd)*Point;
            //}
//////////////////////////////////////////////////////////////////////
         g_trade_mode = 2;
//////////////////////////////////////////////////////////////////////      
      }           
      
      if(phase_profit < 0) {
         cmt = "BUY(duringbuyin(-)) RT " + ticket_phase;
         if(!MoneyManagement) {
            lots = phase_lots + LotsAdd;
         }
	      
            if(StopLossMinusB1L) {
               if(StopLoss-StopLossMinusB1L<6) 
                  {
                     SLMB1L=6;
                  }
               price_stop = price_stop + pips2Points(SLMB1L)*Point;
               Print("B1L     ",price_now - price_stop);
            }                     
   
         if(Previous_TakeProfit) {
            price_profit = phase_price_tp;
         }
         else {
            price_profit = price_profit + pips2Points(TakeProfitAdd)*Point;
         }
//////////////////////////////////////////////////////////////////////
         g_trade_mode = 2;
//////////////////////////////////////////////////////////////////////
         
      }
   }
   
   if(!DD_SameSide && !DD_MapRT && ticket_phase < 0) {
      ticket_phase = findPhaseOrder(OP_SELL);
   int SLMB2P=StopLossMinusB2P;
   int SLMB2L=StopLossMinusB2L;
      if(ticket_phase > 0) {
         
         OrderSelect(ticket_phase, SELECT_BY_TICKET);
         phase_price_op = OrderOpenPrice();
         phase_price_tp = OrderTakeProfit();
         phase_profit = OrderProfit();
         phase_lots = OrderLots();
         phase_time_open = OrderOpenTime();           
      
         if(phase_profit > 0) {
         cmt = "BUY(duringSELLin(+)) RT " + ticket_phase;
         /*if(!MoneyManagement) {
               lots = phase_lots + LotsAdd;
            }*/
            if(StopLossMinusB2P) {
               if(StopLoss-StopLossMinusB2P<6) 
                  {
                     SLMB2P=6;
                  }
               price_stop = price_stop + pips2Points(SLMB2P)*Point;
               Print("B2P     ",price_now - price_stop);
            }                     
            price_profit = price_profit + pips2Points(TakeProfitAdd)*Point;
            
//////////////////////////////////////////////////////////////////////
            g_trade_mode = 2;
//////////////////////////////////////////////////////////////////////
         }           
         
         if(phase_profit < 0) {
            cmt = "BUYduringSELLin(-)) RT " + ticket_phase;
            if(!MoneyManagement) {
               lots = phase_lots + LotsAdd;
            }
            if(StopLossMinusB2L) {
               if(StopLoss-StopLossMinusB2L<6) 
                  {
                     SLMB2L=6;
                  }
               price_stop = price_stop + pips2Points(SLMB2L)*Point;
               Print("B2     ",price_now - price_stop);
            }                     
            price_profit = price_profit + pips2Points(TakeProfitAdd)*Point;
            
//////////////////////////////////////////////////////////////////////
            g_trade_mode = 2;
//////////////////////////////////////////////////////////////////////
            
         }
      }
   }
   
    if (DD_MapRT)
    {
        int ticket_phase2 = ticket_phase;
        ddMapRT(OP_BUY, ticket_phase2);
        if (ticket_phase2 > 0 && ticket_phase2 != ticket_phase)
        {
            //Print(ticket_phase2,"   ","DD_MapRT Buy 3");
            OrderSelect(ticket_phase2, SELECT_BY_TICKET);
            phase_price_op = OrderOpenPrice();
            phase_price_tp = OrderTakeProfit();
            phase_profit = OrderProfit();
            phase_lots = OrderLots();
            phase_time_open = OrderOpenTime();
            cmt = "DDMapRTBUY3";
            if(!MoneyManagement) {
              lots = phase_lots + LotsAdd;
            }

            if(Previous_TakeProfit) {
              price_profit = phase_price_tp;
            }
            else {
              price_profit = price_profit + pips2Points(TakeProfitAdd)*Point;
            }
        }
    }
   
   // Disable the stoploss for Ghost Stoploss feature.
   //if (flag_gsl && cmt == "DD") price_stop = 0; RE-ENABLE IF NEEDED, AS "DD" COMMENT HAS BEEN CHANGED! 11-14-16      
   Log("Buy");
   if(!ECNBroker && !StealthMode) {
      ticket = OrderSend(Symbol(), OP_BUY, lots, price_now, Slippage, price_stop, price_profit, cmt, Magic, 0, Lime);
   }
   else {
      ticket = OrderSend(Symbol(), OP_BUY, lots, price_now, Slippage, 0, 0, cmt, Magic, 0, Lime);
   }
   
   if(ticket < 0) {
      //tradeBuy();
      Print(AdvisorName, "- tradeBuy() error #", GetLastError());
      Print("Price Buy: ", price_now, " TP: ", price_profit, " SL: ", price_stop, " Lots: ", lots);
   }
   else {
      OrderSelect(ticket, SELECT_BY_TICKET);
      fxGV(strTradeType(), SET, g_trade_mode);
      if(ECNBroker) {           
         OrderModify(OrderTicket(), OrderOpenPrice(), price_stop, price_profit, 0, CLR_NONE);
      }
   }
   
   return(ticket);
  }  
  
//+------------------------------------------------------------------+
//| Trade Sell @http://kolier.li @v2.0                               |
//| @import                                                          |
//|   bool ECNBroker                                                 |
//|   double TakeProfit                                              |
//|   double StopLoss                                                |
//|   double Lots                                                    |
//+------------------------------------------------------------------+
int tradeSell(double tp=0, double sl=0, double lots=0)
  {
   double price_now, price_profit, price_stop;
   double take_profit = pips2Points(TakeProfit)*Point;
   if(tp>0) take_profit = tp; 
   double stop_loss = pips2Points(StopLoss)*Point;
   if(sl>0) stop_loss = sl;
   int ticket;
   price_now = Bid;
   if(take_profit > 0) {
      price_profit = price_now - take_profit;
   }
   else {
      price_profit = 0;
   }
   if(stop_loss > 0) {
      price_stop = price_now + stop_loss;
   }
   else {
      price_stop = 0;
   }

   if(MoneyManagement && lots==0) {
      lots = mmLots();
   } 

   if(lots==0) lots = Lots;
   
   //Since the martiangle pramaters may be set in a different tick we don't change it if it was 3.
   
   if(g_trade_mode!=3) {
      g_trade_mode = 1;
      //Print("g4");
   }
   string cmt = "New";
   
   int ticket_phase = findPhaseOrder(OP_SELL);
   double phase_price_op, phase_price_tp, phase_profit, phase_lots;
   int phase_time_open;
   int j;
   int SLMS1P=StopLossMinusS1P;
   int SLMS1L=StopLossMinusS1L;
   if(ticket_phase > 0) {
      
      OrderSelect(ticket_phase, SELECT_BY_TICKET);
      phase_price_op = OrderOpenPrice();
      phase_price_tp = OrderTakeProfit();
      phase_profit = OrderProfit();
      phase_lots = OrderLots();
      phase_time_open = OrderOpenTime();
      
      if(phase_profit > 0) {
      cmt = "SELL(duringSELLin(+)) RT " + ticket_phase;
      /*if(!MoneyManagement) {
            lots = phase_lots + LotsAdd;
         }*/
	      
            if(StopLossMinusS1P) {
               if(StopLoss-StopLossMinusS1P<6) 
                  {
                     SLMS1P=6;
                  }
               price_stop = price_stop - pips2Points(SLMS1P)*Point;
               Print("S1P     ",price_now - price_stop);
            }                     
            //if(Previous_TakeProfit) {
            //price_profit = phase_price_tp;
            //}
            //else {
            price_profit = price_profit - pips2Points(TakeProfitAdd)*Point;
            //}

//////////////////////////////////////////////////////////////////////
         g_trade_mode = 2;
//////////////////////////////////////////////////////////////////////      
      }           
      
      if(phase_profit < 0) {
         cmt = "SELL(duringSELLin(-)) RT " + ticket_phase;
         if(!MoneyManagement) {
            lots = phase_lots + LotsAdd;
         }
	      
            if(StopLossMinusS1L) {
               if(StopLoss-StopLossMinusS1L<6) 
                  {
                     SLMS1L=6;
                  }
               price_stop = price_stop - pips2Points(SLMS1L)*Point;
               Print("S1L     ",price_now - price_stop);
            }                     
            if(Previous_TakeProfit) {
               price_profit = phase_price_tp;
         }
         else {
            price_profit = price_profit - pips2Points(TakeProfitAdd)*Point;
         }
         
//////////////////////////////////////////////////////////////////////
         g_trade_mode = 2;
//////////////////////////////////////////////////////////////////////
         }
   }
   
   if(!DD_SameSide && !DD_MapRT && ticket_phase < 0) {
      ticket_phase = findPhaseOrder(OP_BUY);
   int SLMS2P=StopLossMinusS2P;
   int SLMS2L=StopLossMinusS2L;
      if(ticket_phase > 0) {
         
         //Print(ticket_phase,"   ","DDSell2(During Buy)");
         OrderSelect(ticket_phase, SELECT_BY_TICKET);
         phase_price_op = OrderOpenPrice();
         phase_price_tp = OrderTakeProfit();
         phase_profit = OrderProfit();
         phase_lots = OrderLots();
         phase_time_open = OrderOpenTime();

         if(phase_profit > 0) 
         {
         cmt = "SELL(duringBUYin(+)) RT " + ticket_phase;
         /*if(!MoneyManagement) {
               lots = phase_lots + LotsAdd;
            }*/
            
            if(StopLossMinusS2P) {
               if(StopLoss-StopLossMinusS2P<6) 
                  {
                     SLMS2P=6;
                  }
               price_stop = price_stop - pips2Points(SLMS2P)*Point;
               Print("S2P     ",price_now - price_stop);
            }                     
            price_profit = price_profit - pips2Points(TakeProfitAdd)*Point;
            
//////////////////////////////////////////////////////////////////////
            g_trade_mode = 2;
//////////////////////////////////////////////////////////////////////         
         }           
         
         if(phase_profit < 0) {
            cmt = "SELL(duringBUYin(-)) RT " + ticket_phase;
            if(!MoneyManagement) {
               lots = phase_lots + LotsAdd;
            }
            
            if(StopLossMinusS2L) {
               if(StopLoss-StopLossMinusS2L<6) 
                  {
                     SLMS2L=6;
                  }
               price_stop = price_stop - pips2Points(SLMS2L)*Point;
               Print("S2L     ",price_now - price_stop);
            }                     
            price_profit = price_profit - pips2Points(TakeProfitAdd)*Point;
            
//////////////////////////////////////////////////////////////////////
            g_trade_mode = 2;
//////////////////////////////////////////////////////////////////////
            }
      }
   }
   
   if (DD_MapRT)
   {
        int ticket_phase2 = ticket_phase;
        ddMapRT(OP_SELL, ticket_phase2);
        if (ticket_phase2 > 0 && ticket_phase2 != ticket_phase)
        {
           OrderSelect(ticket_phase2, SELECT_BY_TICKET);
           phase_price_op = OrderOpenPrice();
           phase_price_tp = OrderTakeProfit();
           phase_profit = OrderProfit();
           phase_lots = OrderLots();
           phase_time_open = OrderOpenTime();
           cmt = "DDMapRTSELL3";
           if(!MoneyManagement) {
              lots = phase_lots + LotsAdd;
           }
   
           if(Previous_TakeProfit) {
              price_profit = phase_price_tp;
           }
           else {
              price_profit = price_profit - pips2Points(TakeProfitAdd)*Point;
           }
        }
   }
   
   //if (flag_gsl && cmt == "DD") price_stop = 0; RE-ENABLE IF NEEDED, AS "DD" COMMENT HAS BEEN CHANGED! 11-14-16   
   Log("sell");
   if(!ECNBroker && !StealthMode) {
      ticket = OrderSend(Symbol(), OP_SELL, lots, price_now, Slippage, price_stop, price_profit, cmt, Magic, 0, Red);
   }
   else {
      ticket = OrderSend(Symbol(), OP_SELL, lots, price_now, Slippage, 0, 0, cmt, Magic, 0, Red);
   }
   
   if(ticket < 0) {
      //tradeBuy();
      Print(AdvisorName, "- tradeSell() error #", GetLastError());
      Print("Price Sell: ", price_now, " TP: ", price_profit, " SL: ", price_stop, " Lots: ", lots);
   }
   else {
      OrderSelect(ticket, SELECT_BY_TICKET);
      fxGV(strTradeType(), SET, g_trade_mode);
      if(ECNBroker) {         
         OrderModify(OrderTicket(), OrderOpenPrice(), price_stop, price_profit, 0, CLR_NONE);
      }
   }
   
   return(ticket);
  }
  
//+------------------------------------------------------------------+
//| TP and SL stealthly, hide from the trade server                  |
//| Place at the front part of start()                               |
//+------------------------------------------------------------------+
void stealthMode()
  {
   double takeprofit_stealth = pips2Points(TakeProfit)*Point;
   double stoploss_stealth = pips2Points(StopLoss)*Point;
      
   for(int i=0; i<OrdersTotal(); i++) {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==Magic) {
         if(OrderType()==OP_BUY
            && ( (MarketInfo(OrderSymbol(),MODE_BID)-OrderOpenPrice()>=takeprofit_stealth && takeprofit_stealth>0)
               || (OrderOpenPrice()-MarketInfo(OrderSymbol(),MODE_BID)>=stoploss_stealth && stoploss_stealth>0) )) {
            OrderClose(OrderTicket(), OrderLots(), Bid, Slippage, White);
         }
         else if(OrderType()==OP_SELL
            && ( (OrderOpenPrice()-MarketInfo(OrderSymbol(),MODE_ASK)>=takeprofit_stealth && takeprofit_stealth>0)
               || (MarketInfo(OrderSymbol(),MODE_ASK)-OrderOpenPrice()>=stoploss_stealth && stoploss_stealth>0) )) {
            OrderClose(OrderTicket(), OrderLots(), Ask, Slippage, White);
         }
      }
   }
  }
  
//+------------------------------------------------------------------+
//| MM Lots @http://kolier.li                                        |
//|   Open lots base on the percent of equity and MM_RiskPips        |
//+------------------------------------------------------------------+
double mmLots()
  {
   double lots_equity = AccountEquity() * MM_Risk/100;
   double lots = NormalizeDouble(lots_equity/(MM_RiskPips*pipValue()), MM_LotsPrecision);
   
   return(lots);
  }
  
//+------------------------------------------------------------------+
//| Convert From TickValue @http://kolier.li                         |
//+------------------------------------------------------------------+
double pipValue()
  {
   double tick_size = MarketInfo(Symbol(), MODE_TICKSIZE);
   double tick_pips = points2Pips(tick_size*MathPow(10,MarketInfo(Symbol(),MODE_DIGITS)));
   double tick_value = MarketInfo(Symbol(), MODE_TICKVALUE);
   
   return(tick_value/tick_pips);
  }

//+------------------------------------------------------------------+
//| points2Pips() @http://kolier.li                                  |
//+------------------------------------------------------------------+
double points2Pips(double points)
  {
   double pips;
   if(Close[0]<10) {
      pips = points/MathPow(10,Digits-4);
   }
   else if(Close[0]>10) {
      pips = points/MathPow(10,Digits-2);
   }
   
   return(pips);
  }
  
//+------------------------------------------------------------------+
//| pips2Points() @http://kolier.li                                  |
//+------------------------------------------------------------------+
double pips2Points(double pips)
  {
   double points;
   if(Close[0]<10) {
      points = pips*MathPow(10,Digits-4);
   }
   else if(Close[0]>10) {
      points = pips*MathPow(10,Digits-2);  
   }
   
   return(points);
  }
  
//+------------------------------------------------------------------+
//| Close Buy Trades @http://kolier.li                               |
//+------------------------------------------------------------------+
void closeTradesBuy()
  {
   int trades_total = OrdersTotal();
   for(int i=0; i<trades_total; i++) {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol()==Symbol() && OrderType()==OP_BUY && OrderMagicNumber()==Magic) {
         OrderClose(OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(),MODE_BID), Slippage, White);
      }
   }
  }
  
//+------------------------------------------------------------------+
//| Close Trades Sell @http://kolier.li                              |
//+------------------------------------------------------------------+
void closeTradesSell()
  {
   int trades_total = OrdersTotal();
   for(int i=0; i<trades_total; i++) {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol()==Symbol() && OrderType()==OP_SELL && OrderMagicNumber()==Magic) {
         OrderClose(OrderTicket(), OrderLots(), MarketInfo(OrderSymbol(),MODE_ASK), Slippage, White);
      }
   }
  }

//+------------------------------------------------------------------+
//| trailStop Target Points                                          |
//| Scan Trades @http://kolier.li @v1.0                              |
//| @params                                                          |
//|   string order_type TRADEALL|TRADETRADE|TRADEPEND                |
//|   string order_dir TRADEALL|TRADEBUY|TRADESELL                   |
//| @return                                                          |
//|   int orders_number                                              |
//+------------------------------------------------------------------+
int scanTrades(int order_type=TRADEALL, int order_dir=TRADEALL)
  {
   int orders_total = OrdersTotal();
   int orders_number = 0;
   for(int i=0; i<orders_total; i++)
   {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(OrderMagicNumber() == Magic && OrderSymbol() == Symbol()) {
         switch(order_type) {
            case TRADEALL:
               orders_number++;
               break;
            case TRADETRADE:
               if(OrderType()<=OP_SELL) {
                  orders_number = _scanTradesDir(order_dir, orders_number);
               }
               break;
            case TRADEPEND:
               if(OrderType()>OP_SELL) {
                  orders_number = _scanTradesDir(order_dir, orders_number);
               }
               break;
         }// switch
      }
   }
   return(orders_number);   
  }
  
int _scanTradesDir(int order_dir, int orders_number)
  {
      switch(order_dir) {
         case TRADEALL:
            orders_number++;
            break;
         case TRADEBUY:
            if(OrderType()==OP_BUY || OrderType()==OP_BUYLIMIT || OrderType()==OP_BUYSTOP) {
               orders_number++;
            }
            break;
         case TRADESELL:
            if(OrderType()==OP_SELL || OrderType()==OP_SELLLIMIT || OrderType()==OP_SELLSTOP) {
               orders_number++;
            }
            break;
      }// switch
      return(orders_number);
  }
//**************************************************************************************************
//**************************************************************************************************
bool BOSO(int op_type)
 {
      datetime time_latest_tpM = 0, time_phaseM = EMPTY_VALUE;
      int order_typeM;
      int ticketM = -1;
      int iM;
    
    for (iM = 0; iM < OrdersHistoryTotal(); iM++)
    {
      OrderSelect(iM, SELECT_BY_POS, MODE_HISTORY);
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == Magic && OrderProfit()<0 && OrderLots()==PeakLots)
       {
         if (OrderComment() == "New")
           {
             time_phaseM = OrderOpenTime();
             ticketM = OrderTicket();
             order_typeM = OrderType();
             Print("      CLOSED RT      ",ticketM,"     ",order_typeM, "     ", OrderLots());
            }
        }
    }
            if(MOPP || (!MOPP && !MSAME))
            {
            if(order_typeM == OP_BUY && OrderLots()==PeakLots && OrderProfit()<0)
            {
            BUYONLY=1;Print("MOPP>>>SO, SELL ONLY!!!");
            }
            else 
            if(order_typeM==OP_SELL && OrderLots()==PeakLots && OrderProfit()<0)
            {
            SELLONLY=1;Print("MOPP>>>SO,BUY ONLY!!!");
            }
            if 
            (BUYONLY==1 && op_type==OP_SELL)
            {
            return true;
            }
            else if 
            (SELLONLY==1 && op_type==OP_BUY)
            {
            return true;
            }
            if(BUYONLY==0 && SELLONLY==0)
            {
            return true;
            }
//////////////////////////THE CLINCHER:            
            else if(!MOPP) 
            {
            return true;
            }
            return false;
            }
            
/////////////////////////////////////////////////////////////////////////////////////////////////////
            if(MSAME || (!MSAME && !MOPP))
            {
            if(order_typeM == OP_BUY && OrderLots()==PeakLots && OrderProfit()<0)
            {
            BUYONLY=1;Print("MSAME>>>SO, BUY ONLY!!!");
            }
            else 
            if(order_typeM==OP_SELL && OrderLots()==PeakLots && OrderProfit()<0)
            {
            SELLONLY=1;Print("MSAME>>>SO, SELL ONLY!!!");
            }
            if 
            (BUYONLY==1 && op_type==OP_BUY)
            {
            return true;
            }
            else if 
            (SELLONLY==1 && op_type==OP_SELL)
            {
            return true;
            }
            if(BUYONLY==0 && SELLONLY==0)
            {
            return true;
            }
///////////////////////////THE CLINCHER:           
            else if(!MSAME) 
            {
            return true;
            } 
            return false;
            }
            
 
 }
 
//|---------calculate martingale---ABBREVIATED, REVAMPED 5-27-18 12:44 AM

void CalculateMartingale()
  {
   double InitialLots=InitialLots;
   double MinLots=MarketInfo(Symbol(),MODE_MINLOT);
   double MaxLots=MarketInfo(Symbol(),MODE_MAXLOT);
   double internalMinProfit;

   if(Lots<PeakLots)
   {
      internalMinProfit = MinProfit;
   }
      else if (Lots>=PeakLots)
   {
      internalMinProfit = MinProfit2;
   }
         //Alternate=true;
         //MSAME=true;
         //MOPP=true;
         
         //Alternate=false;
         //MSAME=false;
         //MOPP=false;
//**********************************************************************   
/*   if(Lots > .01 && Lots <= PeakApproach1)
   {     
         if(SecondWind==1){
         StopLoss=LoStopLoss*SuperPeakSLMult;
         TakeProfit=LoTakeProfit*SuperPeakTPMult;         
         //Alternate=true;
         //MSAME=true;
         //MOPP=true;
         
         //Alternate=false;
         //MSAME=false;
         //MOPP=false;
         }
         else{
         StopLoss=LoStopLoss;
         TakeProfit=LoTakeProfit;
         //Alternate=true;
         //MSAME=true;
         //MOPP=true;
         
         //Alternate=false;
         //MSAME=false;
         //MOPP=false;
         }
         Multiplier=NewMultiplier1;
              
   }
//**********************************************************************9-9-'15     
   if(Lots > PeakApproach1 && Lots <= PeakApproach2)
   {
         if(SecondWind==1){
         StopLoss=MidStopLoss*SuperPeakSLMult;
         TakeProfit=MidTakeProfit*SuperPeakTPMult;         
         //Alternate=true;
         //MSAME=true;
         //MOPP=true;
         
         //Alternate=false;
         //MSAME=false;
         //MOPP=false;
         }
         else{
         StopLoss=MidStopLoss;
         TakeProfit=MidTakeProfit;
         //Alternate=true;
         //MSAME=true;
         //MOPP=true;
         
         //Alternate=false;
         //MSAME=false;
         //MOPP=false;
         }         
         Multiplier=NewMultiplier2;
         
   }
//********************************************************************** 
   if(Lots > PeakApproach2 && Lots <= PeakApproach3)
   {
         if(SecondWind==1){
         StopLoss=HiStopLoss*SuperPeakSLMult;
         TakeProfit=HiTakeProfit*SuperPeakTPMult;
         //Alternate=true;
         //MSAME=true;
         //MOPP=true;
         
         //Alternate=false;
         //MSAME=false;
         //MOPP=false;
         }
         else{
         StopLoss=HiStopLoss;
         TakeProfit=HiTakeProfit;
         //Alternate=true;
         //MSAME=true;
         //MOPP=true;
         
         //Alternate=false;
         //MSAME=false;
         //MOPP=false;
         }
         Multiplier=NewMultiplier3;
            
   }
//**********************************************************************12-7-'15

   if(Lots > PeakApproach3 && Lots <= PeakApproach4)
   {     
         if(SecondWind==1){
         StopLoss=SupremeStopLoss*SuperPeakSLMult;
         TakeProfit=SupremeTakeProfit*SuperPeakTPMult;
         //Alternate=true;
         //MSAME=true;
         //MOPP=true;
         
         //Alternate=false;
         //MSAME=false;
         //MOPP=false;
         }
         else{
         StopLoss=SupremeStopLoss;
         TakeProfit=SupremeTakeProfit;
         //Alternate=true;
         //MSAME=true;
         //MOPP=true;
         
         //Alternate=false;
         //MSAME=false;
         //MOPP=false;
         }
         Multiplier=NewMultiplier4;
                  
   }*/
//**********************************************************************12-8-'15***
   if(Lots > PeakLots)
   { 
         if(SecondWind==1){
         Lots=PeakLots*(PeakLotsMult);//********************
         StopLoss=SuperStopLoss*SuperPeakSLMult;
         TakeProfit=SuperTakeProfit*SuperPeakTPMult;
         //Alternate=true;
         //MSAME=true;
         //MOPP=true;
         
         //Alternate=false;
         //MSAME=false;
         //MOPP=false;
         }
         else{
         StopLoss=SuperStopLoss;//**************************
         TakeProfit=SuperTakeProfit;//**********************
         Lots=PeakLots;
         //Alternate=true;
         //MSAME=true;
         //MOPP=true;
         
         //Alternate=false;
         //MSAME=false;
         //MOPP=false;
         }
         
         
   }

   
   if(Balance!=0.0)
         {
            if(Balance>AccountBalance())        //this is the case when there has not been a profit
            
            {
            
            Lots=Multiplier*Lots;
                           //this is the correct place where it has to happen - the "loss case"

            Print("SL=   ",MathAbs((OrderOpenPrice()-OrderStopLoss())*10000));
            Print("TP=   ",MathAbs((OrderOpenPrice()-OrderTakeProfit())*10000));
            g_trade_mode = 3;                                
            
            Print(SecondWind);
            }         
      
      else if((Balance+internalMinProfit)<AccountBalance()) // we made profit -> reset
            
            {     
               Lots=InitialLots;           //resetting TP and SL here
               TakeProfit=origTakeProfit;  //look at the first few lines, we stored StopLoss and TakeProfit 
               StopLoss=origStopLoss;      //now to two new variables
               Multiplier=origMultiplier;
               
               SecondWind=0;
               
               BUYONLY=0;
               SELLONLY=0;
               
               //Alternate=false; SOMEDAY, 'MALT'?
               MSAME=false;
               MOPP=false;
               //keepon=0; IRRELEVANT ANYMORE
               
               Print("SL=   ",MathAbs((OrderOpenPrice()-OrderStopLoss())*10000));
               Print("TP=   ",MathAbs((OrderOpenPrice()-OrderTakeProfit())*10000));                             
               g_trade_mode = 1;
               
               Print(SecondWind);
               
            }
      else if((Balance+internalMinProfit)>=AccountBalance()) 
               // nearly same as first one - this is we made profit, but min profit isn't reached yet
            {
               //here we don't change g_trade_mode. It should keep its value in this case.
               Lots=Lots;                              
            }
            if(Lots<MinLots)Lots=MinLots;
            if(Lots>MaxLots)Lots=MaxLots;
            }
    return(0);
  }

//************************************************************************************************** 
//+------------------------------------------------------------------+
//| Break Even @http://kolier.li                                     |
//+------------------------------------------------------------------+
void breakEven()
  {
   int orders_total = OrdersTotal();
   int i, ticket;
   double break_even_points = pips2Points(BreakEvenPips)*Point;
   double break_even_with_profit = pips2Points(BreakEvenProfit)*Point;
   double stoploss_buy, stoploss_sell;
   string gv_name;
   double break_even_able;
   double ask_s, bid_s;
   
   for(i=0; i<orders_total; i++) {
      break_even_able = -1;
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      ask_s = MarketInfo(OrderSymbol(), MODE_ASK);
      bid_s = MarketInfo(OrderSymbol(), MODE_BID);
      if(!BreakEvenWithProfit) {
         stoploss_buy = OrderOpenPrice();
         stoploss_sell = OrderOpenPrice();
      }
      else {
         stoploss_buy = OrderOpenPrice() + break_even_with_profit;
         stoploss_sell = OrderOpenPrice() - break_even_with_profit;
      }
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==Magic && OrderProfit()>0) {
         if(BE_StealthMode) {
            break_even_able = fxGV(AdvisorName+"_be_able_"+OrderTicket());
            if(OrderType()==OP_BUY && break_even_able>0 && bid_s<=stoploss_buy) {
               OrderClose(OrderTicket(), OrderLots(), bid_s, Slippage, White);
            }
            else if(OrderType()==OP_SELL && break_even_able>0 && ask_s>=stoploss_sell) {
               OrderClose(OrderTicket(), OrderLots(), ask_s, Slippage, White);
            }
         }
      
         if(OrderType()==OP_BUY && (OrderStopLoss()<stoploss_buy || OrderStopLoss()==0)
            && bid_s-OrderOpenPrice()>=break_even_points) {
            if(!BE_StealthMode) {
               OrderModify(OrderTicket(), OrderOpenPrice(), stoploss_buy, OrderTakeProfit(), 0, Green);
            }
            else {
               fxGV(AdvisorName+"_be_able_"+OrderTicket(), SET, 1);
            }
         }
         else if(OrderType()==OP_SELL && (OrderStopLoss()>stoploss_sell || OrderStopLoss()==0)
            && OrderOpenPrice()-ask_s>=break_even_points) {
            if(!BE_StealthMode) {
               OrderModify(OrderTicket(), OrderOpenPrice(), stoploss_sell, OrderTakeProfit(), 0, Magenta);
            }
            else {
               fxGV(AdvisorName+"_be_able_"+OrderTicket(), SET, 1);
            }
         }
      }
   }
  }

//string s_ts_state_str = "_tsstate_";
bool getTrailingStopParams(string comment, int& TS_Pips, int& TS_MovingRange, int& TS_StartPips, bool& Use_Ts) {
   //int pos = StringFind(comment,s_ts_state_str); 
   //if(pos==-1) return false;
   //string cut_st = StringSubstr(comment, pos+StringLen(s_ts_state_str));
   int mode = fxGV(strTradeType());
   if(mode == 3) {
      TS_Pips = TS_Pips3;
      TS_MovingRange = TS_MovingRange3;
      TS_StartPips = TS_StartPips3;
      Use_Ts = UseTrailStop3;
   } else if(mode==2) {
      TS_Pips = TS_Pips2;
      TS_MovingRange = TS_MovingRange2;
      TS_StartPips = TS_StartPips2;
      Use_Ts = UseTrailStop2;
   } else {
      TS_Pips = TS_Pips1;
      TS_MovingRange = TS_MovingRange1;
      TS_StartPips = TS_StartPips1;
      Use_Ts = UseTrailStop1;
   }   
   //Print("mode="+mode);
   return true;
}

//+------------------------------------------------------------------+
//| Trail Stop @http://kolier.li @v2.0                               |
//+------------------------------------------------------------------+
void trailStop()
  {
   int ticket;
   string gv_name;
   double stoploss_order;
   double ask_s, bid_s;
   int TS_Pips, TS_MovingRange, TS_StartPips;
   bool Use_Ts;
      
   for(int i=0; i<OrdersTotal(); i++) {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      ask_s = MarketInfo(OrderSymbol(), MODE_ASK);
      bid_s = MarketInfo(OrderSymbol(), MODE_BID);
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==Magic) {
         if(!getTrailingStopParams(OrderComment(), TS_Pips, TS_MovingRange, TS_StartPips, Use_Ts)) continue;
         if(!Use_Ts) continue;
         double ts_points = pips2Points(TS_Pips)*Point;
         double ts_movingrange = pips2Points(TS_MovingRange)*Point;
         double ts_start = pips2Points(TS_StartPips)*Point;
         
         if(!TS_StealthMode) {
            stoploss_order = OrderStopLoss();
         }
         else {
            stoploss_order = fxGV(AdvisorName+"_ts_stoploss_"+OrderTicket());
            
            if(stoploss_order>0) {
               if(OrderType()==OP_BUY && bid_s<=stoploss_order) {
                  OrderClose(OrderTicket(), OrderLots(), bid_s, Slippage, White);
                  fxGV(AdvisorName+"_ts_stoploss_"+OrderTicket(), DEL);
                  continue;
               }
               else if(OrderType()==OP_SELL && ask_s>=stoploss_order) {
                  OrderClose(OrderTicket(), OrderLots(), ask_s, Slippage, White);
                  fxGV(AdvisorName+"_ts_stoploss_"+OrderTicket(), DEL);
                  continue;
               }
            }
         }

         if(OrderType()==OP_BUY
            && ((bid_s-ts_points>stoploss_order && stoploss_order>0) || stoploss_order==0)
            && bid_s-OrderOpenPrice()>=ts_start
            && ((bid_s-stoploss_order>ts_points+ts_movingrange && stoploss_order>0) || stoploss_order==0)
         ) {
            if(!TS_StealthMode) {
               OrderModify(OrderTicket(), OrderOpenPrice(), bid_s-ts_points, OrderTakeProfit(), 0, Blue);
            }
            else {
               fxGV(AdvisorName+"_ts_stoploss_"+OrderTicket(), SET, bid_s-ts_points);
            }
         }
         else if(OrderType()==OP_SELL
            && ((ask_s+ts_points<stoploss_order && stoploss_order>0) || stoploss_order==0)
            && OrderOpenPrice()-ask_s>=ts_start
            && ((stoploss_order-ask_s>ts_points+ts_movingrange && stoploss_order>0) || stoploss_order==0)
         ) {
            if(!TS_StealthMode) {
               OrderModify(OrderTicket(), OrderOpenPrice(), ask_s+ts_points, OrderTakeProfit(), 0, Orange);
            }
            else {
               fxGV(AdvisorName+"_ts_stoploss_"+OrderTicket(), SET, ask_s+ts_points);
            }
         }
      }
   }
  }
  
//+------------------------------------------------------------------+
//| Explode String @http://kolier.li                                 |
//+------------------------------------------------------------------+  
void explodeStr2Points(double& arr[], string text_arr, string separator=",")
  {
    int    start = 0,
           length,
           array_size;
    bool   eos = false;
    
    ArrayResize(arr, 0);
    
    while(eos==false) {
      length = StringFind(text_arr,separator,start)-start;
      if(length < 0) {
         // Avoid empty string
         if(StringLen(text_arr)==0) {
            return;
         }
         eos = true;
         length = StringLen(text_arr)-start;
      }
      
      array_size = ArraySize(arr);
      ArrayResize(arr, array_size+1);
      arr[array_size] = pips2Points(StrToDouble(StringSubstr(text_arr, start, length)))*Point;
      start += length+1;
    }
  }
  
/**
 * Find the latest closed order.
 * @author Kolier Li
 * @url http://kolierfx.com
 */
int fxOrderFindLatestClosed(string symbol="", int magic = 0, int op_type = -1, string cmt = "")
{
    datetime time_closed = 0;
    int ticket = -1;
    int i;
    for (i = OrdersHistoryTotal() - 1; i >= 0; i--)
    {
        OrderSelect(i, SELECT_BY_POS, MODE_HISTORY);
        if (
            (StringLen(symbol) == 0 || OrderSymbol() == symbol)
            && OrderMagicNumber() == magic
            && (op_type == -1 || OrderType() == op_type)
            && (cmt == "" || StringFind(OrderComment(), cmt) != -1)
            && OrderCloseTime() > time_closed
        )
        {
            time_closed = OrderCloseTime();
            ticket = OrderTicket();
        }
    }
    
    return (ticket);
}

/**
 * Global Variable operation.
 */
double fxGV(string name, int op = GET, double value = 0)
{
    string name_real = name;
    
    if (IsTesting())
    {
        name_real = "(Test)" + name_real;
    }

    switch (op)
    {
        case GET:
            return(GlobalVariableGet(name_real));
            break;
        case SET:
            GlobalVariableSet(name_real, value);
            return(value);
            break;
        case DEL:
            GlobalVariableDel(name_real);
            break;
    }

    return(value);
}

double fxOrderVOpenPrice(int ticket)
{
    if (OrderSelect(ticket, SELECT_BY_TICKET) == false) return(-1);
    return(OrderOpenPrice());
}
/////////////////////////////////////////////////////////////////////////////////////////////////
